#include <stdlib.h>
#include <stdbool.h>
#include <Ecore.h>

#include "core/log.h"
#include "core/list.h"
#include "core/config-parser.h"
#include "standard-vibcore.h"

#define VIBRATION_CONF_PATH        "/usr/share/feedback/vibration.conf"

struct vibration_config {
        char *pattern;
        dd_list *data;
};

struct duration_data {
        int duration;
        int wait;
};

struct haptic_data {
        dd_list *vibration_data;
        unsigned int handle;
        int level;
        int priority;
        bool stop;
};

static dd_list *vib_conf_list;
static Ecore_Timer *duration_timer;

static t_vibrate_monotone real_vibrate_monotone;

static int vibration_load_config(struct parse_result *result, void *user_data)
{
	struct vibration_config *conf;
	struct duration_data *data;
	char *value;
	char *check;
	int len;

	if (!result)
		return 0;

	if (!MATCH(result->section, "Vibration"))
		return 0;


	if (!result->name || !result->value)
		return 0;

	conf = (struct vibration_config *)calloc(1, sizeof(struct vibration_config));
	if (!conf) {
		_E("fail to alloc");
		return -ENOMEM;
	}

	conf->pattern = strdup(result->name);
	if (!conf->pattern)
		_E("fail to copy %s pattern data", result->name);

	value = result->value;

	if (!value)
		len = 0;
	else
		len = strlen(value);

	if (len == 0) {
		data = (struct duration_data *)malloc(sizeof(struct duration_data));
		if (!data) {
			_E("not enough memory");
			free(conf->pattern);
			free(conf);
			return -ENOMEM;
		}
		data->duration = 0;
		data->wait = 0;

		DD_LIST_APPEND(conf->data, data);
		DD_LIST_APPEND(vib_conf_list, conf);
		return 0;
	}

	do {
		data = (struct duration_data *)malloc(sizeof(struct duration_data));
		if (!data) {
			_E("not enough memory");
			free(conf->pattern);
			free(conf);
			return -ENOMEM;
		}
		data->duration = 0;
		data->wait = 0;

		check = strchr(value, 'D');
		if (check) {
			*check = '\0';
			data->duration = strtol(value, NULL, 10);
			if (!value)
				len = len - 1;
			else
				len = len - strlen(value) - 1;
			value = check + 1;
		}
		check = strchr(value, 'W');
		if (check) {
			*check = '\0';
			data->wait = strtol(value, NULL, 10);
			if (!value)
				len = len - 1;
			else
				len = len - strlen(value) - 1;
			value = check + 1;
		}
		DD_LIST_APPEND(conf->data, data);
		if (data->duration == 0 && data->wait == 0)
			break;
	} while (value && len > 0);

	DD_LIST_APPEND(vib_conf_list, conf);

	return 0;
}

int standard_config_parse()
{
	return config_parse(VIBRATION_CONF_PATH, vibration_load_config, NULL);
}

int standard_is_supported(const char *pattern)
{
        dd_list *elem;
        struct vibration_config *conf;
        size_t len;
        int ret;

        if (!pattern)
                return -EINVAL;

        len = strlen(pattern) + 1;
        ret = 0;
        DD_LIST_FOREACH(vib_conf_list, elem, conf) {
                if (!conf->pattern)
                        continue;
                if (!strncmp(conf->pattern, pattern, len)) {
                        ret = true;
                        break;
                }
        }

        return ret;
}

static Eina_Bool haptic_duration_play(void *data)
{
	dd_list *head, *n, *next;
	struct haptic_data *h_data;
	struct duration_data *node;
	int ret = 0;

	if (!data)
		goto out;

	if (duration_timer) {
		ecore_timer_del(duration_timer);
		duration_timer = NULL;
	}

	h_data = (struct haptic_data *)data;
	if (h_data->stop) {
		h_data->stop = false;
		free(h_data);
		goto out;
	}

	head = h_data->vibration_data;
	DD_LIST_FOREACH_SAFE(head, n, next, node) {
		_D("Play: %dms and Wait: %dms", node->duration, node->wait);
		if (!node->duration) {
			free(h_data);
			break;
		}

		if (node->wait && next) {
			h_data->vibration_data = next;
			duration_timer = ecore_timer_add((node->duration + node->wait)/1000.0f, haptic_duration_play, (void *)h_data);
		}

		ret = real_vibrate_monotone(h_data->handle, node->duration, h_data->level, h_data->priority, NULL);
		if (!next) {
			free(h_data);
			goto out;
		}
		break;
	}
	if (ret != 0) {
		_D("auto stop vibration");
		if (h_data)
			h_data->stop = true;
	}
out:
	return ECORE_CALLBACK_CANCEL;
}

int standard_set_vib_function(t_vibrate_monotone func)
{
	real_vibrate_monotone = func;
	return 0;
}

int standard_vibrate_effect(int device_handle, const char *pattern, int feedback, int priority)
{
        dd_list *elem;
        struct vibration_config *conf;
        struct haptic_data *data;
        size_t len;

        if (device_handle < 0)
                return -EINVAL;

        len = strlen(pattern) + 1;
        DD_LIST_FOREACH(vib_conf_list, elem, conf) {
                if (!conf->pattern)
                        continue;
                if (strncmp(conf->pattern, pattern, len))
                        continue;

                data = (struct haptic_data *)malloc(sizeof(struct haptic_data));
                if (!data) {
                        _E("fail to alloc");
                        return -ENOMEM;
                }
                data->vibration_data = conf->data;
                data->handle = device_handle;
                data->level = feedback;
                data->priority = priority;
                data->stop = false;
                _D("Play %s", conf->pattern);
                haptic_duration_play((void *)data);
                break;
        }

        return 0;
}

int standard_vibrate_close()
{
	if (duration_timer) {
		ecore_timer_del(duration_timer);
		duration_timer = NULL;
	}

	return 0;
}

